/* Copyright (C) 
 * 2014 - Wayne Ren
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

/*
 *		tool chain dependent part of t_stddef.h (for ccac)
 */

#ifndef TOPPERS_TOOL_STDDEF_H
#define TOPPERS_TOOL_STDDEF_H

/*
 *  macro definitions of compiler extend function
 */
#ifndef __cplusplus					/* C++ supports inline */
#if __STDC_VERSION__ < 199901L		/* C99 supports inline  */
#define inline	__inline__			/* inline function */
#endif /* __STDC_VERSION__ < 199901L */
#endif /* __cplusplus */

#define Inline	static __inline__	/* inline function */

#ifndef __cplusplus					/* C++ supports asm */
#define asm		__asm__				/* inline asm */
#endif /* __cplusplus */

#define Asm		__asm__ volatile	/* inline asm (no optimization) */

#define NoReturn	__attribute__((__noreturn__))
									/* function without return */

/*
 *  utilize the standard header files of tool chain 
 *
 *  the definition of NULL is in stddef.h, INT_MAX, INT_MIN, UINT_MAX, LONG_MAX, 
 *  LONG_MIN, ULONG_MAX, CHAR_BIT come from limits.h.
 *
 *  for C++/EC++, these header files may not be the standard header
 *  files.(for most tool chain, these header files are supported)
 *
 */
#ifndef TOPPERS_MACRO_ONLY
#include <stddef.h>
#include <limits.h>
#endif /* TOPPERS_MACRO_ONLY */

/*
 *  definitions for float 
 *
 *  TOPPERS_STDFLOAT_TYPE1: float is IEEE754 compatible
 *							double is double float
 */
#ifdef TOPPERS_STDFLOAT_TYPE1
#ifndef TOPPERS_MACRO_ONLY

typedef float		float32_t;			/* IEEE754 compatible float */
typedef double		double64_t;			/* IEEE754 compatible double float */

#endif /* TOPPERS_MACRO_ONLY */

#define FLOAT32_MIN		1.17549435e-38F
#define FLOAT32_MAX		3.40282347e+38F
#define DOUBLE64_MIN	2.2250738585072014e-308
#define DOUBLE64_MAX	1.7976931348623157e+308

#endif /* TOPPERS_STDFLOAT_TYPE1 */
#endif /* TOPPERS_TOOL_STDDEF_H */
